using System.IO;
using System.Linq;
using Godot;

namespace KeepersCompound.TMV.UI;

public partial class MissionSelector : Control
{
	[Signal]
	public delegate void LoadMissionEventHandler(string rootPath, string missionPath);

	private InstallPaths _installPaths;

	private FileDialog _FolderSelect;
	private LineEdit _FolderPath;
	private Button _BrowseButton;
	private ItemList _Campaigns;
	private ItemList _Missions;
	private Button _LoadButton;
	private Button _CancelButton;

	public override void _Ready()
	{
		// TODO: Load initial folderpath from config and prefil everything

		_FolderSelect = GetNode<FileDialog>("%FolderSelect");
		_FolderPath = GetNode<LineEdit>("%FolderPath");
		_BrowseButton = GetNode<Button>("%BrowseButton");
		_Campaigns = GetNode<ItemList>("%Campaigns");
		_Missions = GetNode<ItemList>("%Missions");
		_LoadButton = GetNode<Button>("%LoadButton");
		_CancelButton = GetNode<Button>("%CancelButton");

		_BrowseButton.Pressed += () => _FolderSelect.Visible = true;
		_FolderSelect.DirSelected += (string dir) => { _FolderPath.Text = dir; BuildCampaignList(dir); };
		_FolderPath.TextSubmitted += BuildCampaignList;
		_Missions.ItemSelected += (long _) => _LoadButton.Disabled = false;
		_Campaigns.ItemSelected += BuildMissionList;
		_LoadButton.Pressed += EmitLoadMission;
		_CancelButton.Pressed += () => Visible = false;
	}

	public override void _Input(InputEvent @event)
	{
		if (@event is InputEventKey keyEvent && keyEvent.Pressed)
		{
			if (keyEvent.Keycode == Key.Escape)
			{
				Visible = !Visible;
			}
		}
	}

	private void BuildCampaignList(string path)
	{
		_installPaths = new InstallPaths(path);

		_Campaigns.Clear();
		_Missions.Clear();
		_LoadButton.Disabled = true;

		_Campaigns.AddItem("Original Missions");
		var paths = Directory.GetDirectories(_installPaths.fmsPath);
		foreach (var c in paths.OrderBy(s => s))
		{
			_Campaigns.AddItem(c.TrimPrefix(_installPaths.fmsPath));
		}
	}

	private void BuildMissionList(long idx)
	{
		_Missions.Clear();
		_LoadButton.Disabled = true;

		var campaignPath = "";
		if (idx == 0)
		{
			campaignPath = _installPaths.omsPath;
		}
		else
		{
			campaignPath = _installPaths.fmsPath + _Campaigns.GetItemText((int)idx);
		}

		string[] extensions = { "mis", "cow" };
		var paths = Directory.GetFiles(campaignPath);
		foreach (var f in paths.OrderBy(s => s))
		{
			if (extensions.Contains(f.GetExtension().ToLower()))
			{
				_Missions.AddItem(f.TrimPrefix(campaignPath));
			}
		}
	}

	private void EmitLoadMission()
	{
		var selectedCampaign = _Campaigns.GetSelectedItems()[0];
		var selected = _Missions.GetSelectedItems();
		if (selected.IsEmpty())
		{
			return;
		}

		var campaignPath = selectedCampaign == 0 ? _installPaths.omsPath : _installPaths.fmsPath + _Campaigns.GetItemText(selectedCampaign);
		var path = campaignPath + _Missions.GetItemText(selected[0]);
		EmitSignal(SignalName.LoadMission, _installPaths.rootPath, path);

		Visible = false;
	}
}
